@@ -8,13 +8,14 @@ that allows you to interact with the stream data in your own application.
 
 
 __title__ = "livestreamer"
-__version__ = "1.4.3"
+__version__ = "1.4.4"
 __license__ = "Simplified BSD"
 __author__ = "Christopher Rosell"
 __copyright__ = "Copyright 2011-2013 Christopher Rosell"
 __credits__ = ["Christopher Rosell", "Athanasios Oikonomou",
                "Gaspard Jankowiak", "Dominik Dabrowski",
-               "Toad King", "Niall McAndrew", "Daniel Wallace"]
+               "Toad King", "Niall McAndrew", "Daniel Wallace",
+               "Sam Edwards"]
 
 
 from .exceptions import (PluginError, NoStreamsError,
